@@ -30,6 +30,13 @@ class Connector(object):
         self._verify = verify
         self._session = requests.Session()
         self._session.verify = self._verify
+
+        # NOTE(etingof): field studies reveal that some BMCs choke at
+        # long-running persistent HTTP connections (or TCP connections).
+        # By default, we ask HTTP server to shut down HTTP connection we've
+        # just used.
+        self._session.headers['Connection'] = 'close'
+
         if username or password:
             LOG.warning('Passing username and password to Connector is '
                         'deprecated. Authentication is passed through '
@@ -71,6 +78,9 @@ class Connector(object):
         :raises: HTTPError
         """
         url = parse.urljoin(self._url, path)
+        headers = headers or {}
+        if not any(k.lower() == 'odata-version' for k in headers):
+            headers['OData-Version'] = '4.0'
         # TODO(lucasagomes): We should mask the data to remove sensitive
         # information
         LOG.debug('HTTP request: %(method)s %(url)s; headers: %(headers)s; '
