@@ -13,6 +13,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 import logging
+import pkg_resources
+import requests
 
 from sushy import auth as sushy_auth
 from sushy import connector as sushy_connector
@@ -20,15 +22,20 @@ from sushy import exceptions
 from sushy.resources import base
 from sushy.resources.chassis import chassis
 from sushy.resources.compositionservice import compositionservice
+from sushy.resources.fabric import fabric
 from sushy.resources.manager import manager
+from sushy.resources.registry import message_registry
 from sushy.resources.registry import message_registry_file
 from sushy.resources.sessionservice import session
 from sushy.resources.sessionservice import sessionservice
 from sushy.resources.system import system
 from sushy.resources.updateservice import updateservice
+from sushy import utils
 
 LOG = logging.getLogger(__name__)
 
+STANDARD_REGISTRY_PATH = 'standard_registries/'
+
 
 class ProtocolFeaturesSupportedField(base.CompositeField):
 
@@ -79,6 +86,9 @@ class Sushy(base.ResourceBase):
     _chassis_path = base.Field(['Chassis', '@odata.id'])
     """ChassisCollection path"""
 
+    _fabrics_path = base.Field(['Fabrics', '@odata.id'])
+    """FabricCollection path"""
+
     _session_service_path = base.Field(['SessionService', '@odata.id'])
     """SessionService path"""
 
@@ -90,7 +100,9 @@ class Sushy(base.ResourceBase):
 
     def __init__(self, base_url, username=None, password=None,
                  root_prefix='/redfish/v1/', verify=True,
-                 auth=None, connector=None):
+                 auth=None, connector=None,
+                 public_connector=None,
+                 language='en'):
         """A class representing a RootService
 
         :param base_url: The base URL to the Redfish controller. It
@@ -109,6 +121,10 @@ class Sushy(base.ResourceBase):
             the certificates in the directory. Defaults to True.
         :param auth: An authentication mechanism to utilize.
         :param connector: A user-defined connector object. Defaults to None.
+        :param public_connector: A user-defined connector to use for requests
+            on the Internet, e.g., for Message Registries. Defaults to None.
+        :param language: RFC 5646 language code for Message Registries.
+            Defaults to 'en'.
         """
         self._root_prefix = root_prefix
         if (auth is not None and (password is not None or
@@ -123,6 +139,8 @@ class Sushy(base.ResourceBase):
         super(Sushy, self).__init__(
             connector or sushy_connector.Connector(base_url, verify=verify),
             path=self._root_prefix)
+        self._public_connector = public_connector or requests
+        self._language = language
         self._base_url = base_url
         self._auth = auth
         self._auth.set_context(self, self._conn)
@@ -138,9 +156,15 @@ class Sushy(base.ResourceBase):
                             'with %s: %s', self._base_url, ex)
             self._auth = None
 
-    def _parse_attributes(self):
-        super(Sushy, self)._parse_attributes()
-        self.redfish_version = self.json.get('RedfishVersion')
+    def _parse_attributes(self, json_doc):
+        """Parse the attributes of a resource.
+
+        Parsed JSON fields are set to `self` as declared in the class.
+
+        :param json_doc: parsed JSON document in form of Python types
+        """
+        super(Sushy, self)._parse_attributes(json_doc)
+        self.redfish_version = json_doc.get('RedfishVersion')
 
     def get_system_collection(self):
         """Get the SystemCollection object
@@ -153,8 +177,10 @@ class Sushy(base.ResourceBase):
             raise exceptions.MissingAttributeError(
                 attribute='Systems/@odata.id', resource=self._path)
 
-        return system.SystemCollection(self._conn, self._systems_path,
-                                       redfish_version=self.redfish_version)
+        return system.SystemCollection(
+            self._conn, self._systems_path,
+            redfish_version=self.redfish_version,
+            registries=self.registries)
 
     def get_system(self, identity):
         """Given the identity return a System object
@@ -163,7 +189,8 @@ class Sushy(base.ResourceBase):
         :returns: The System object
         """
         return system.System(self._conn, identity,
-                             redfish_version=self.redfish_version)
+                             redfish_version=self.redfish_version,
+                             registries=self.registries)
 
     def get_chassis_collection(self):
         """Get the ChassisCollection object
@@ -177,7 +204,8 @@ class Sushy(base.ResourceBase):
                 attribute='Chassis/@odata.id', resource=self._path)
 
         return chassis.ChassisCollection(self._conn, self._chassis_path,
-                                         redfish_version=self.redfish_version)
+                                         redfish_version=self.redfish_version,
+                                         registries=self.registries)
 
     def get_chassis(self, identity):
         """Given the identity return a Chassis object
@@ -186,7 +214,33 @@ class Sushy(base.ResourceBase):
         :returns: The Chassis object
         """
         return chassis.Chassis(self._conn, identity,
-                               redfish_version=self.redfish_version)
+                               redfish_version=self.redfish_version,
+                               registries=self.registries)
+
+    def get_fabric_collection(self):
+        """Get the FabricCollection object
+
+        :raises: MissingAttributeError, if the collection attribute is
+            not found
+        :returns: a FabricCollection object
+        """
+        if not self._fabrics_path:
+            raise exceptions.MissingAttributeError(
+                attribute='Fabrics/@odata.id', resource=self._path)
+
+        return fabric.FabricCollection(self._conn, self._fabrics_path,
+                                       redfish_version=self.redfish_version,
+                                       registries=self.registries)
+
+    def get_fabric(self, identity):
+        """Given the identity return a Fabric object
+
+        :param identity: The identity of the Fabric resource
+        :returns: The Fabric object
+        """
+        return fabric.Fabric(self._conn, identity,
+                             redfish_version=self.redfish_version,
+                             registries=self.registries)
 
     def get_manager_collection(self):
         """Get the ManagerCollection object
@@ -200,7 +254,8 @@ class Sushy(base.ResourceBase):
                 attribute='Managers/@odata.id', resource=self._path)
 
         return manager.ManagerCollection(self._conn, self._managers_path,
-                                         redfish_version=self.redfish_version)
+                                         redfish_version=self.redfish_version,
+                                         registries=self.registries)
 
     def get_manager(self, identity):
         """Given the identity return a Manager object
@@ -209,7 +264,8 @@ class Sushy(base.ResourceBase):
         :returns: The Manager object
         """
         return manager.Manager(self._conn, identity,
-                               redfish_version=self.redfish_version)
+                               redfish_version=self.redfish_version,
+                               registries=self.registries)
 
     def get_session_service(self):
         """Get the SessionService object
@@ -232,8 +288,9 @@ class Sushy(base.ResourceBase):
         :param identity: The identity of the session resource
         :returns: The Session object
         """
-        return session.Session(self._conn, identity,
-                               redfish_version=self.redfish_version)
+        return session.Session(
+            self._conn, identity,
+            redfish_version=self.redfish_version, registries=self.registries)
 
     def get_update_service(self):
         """Get the UpdateService object
@@ -246,7 +303,8 @@ class Sushy(base.ResourceBase):
 
         return updateservice.UpdateService(
             self._conn, self._update_service_path,
-            redfish_version=self.redfish_version)
+            redfish_version=self.redfish_version,
+            registries=self.registries)
 
     def _get_registry_collection(self):
         """Get MessageRegistryFileCollection object
@@ -274,6 +332,54 @@ class Sushy(base.ResourceBase):
             raise exceptions.MissingAttributeError(
                 attribute='CompositionService/@odata.id',
                 resource=self._path)
+
         return compositionservice.CompositionService(
             self._conn, self._composition_service_path,
-            redfish_version=self.redfish_version)
+            redfish_version=self.redfish_version,
+            registries=self.registries)
+
+    def _get_standard_message_registry_collection(self):
+        """Load packaged standard message registries
+
+        :returns: list of MessageRegistry
+        """
+
+        message_registries = []
+        resource_package_name = __name__
+        for json_file in pkg_resources.resource_listdir(
+            resource_package_name, STANDARD_REGISTRY_PATH):
+                # Not using path.join according to pkg_resources docs
+                mes_reg = message_registry.MessageRegistry(
+                    None, STANDARD_REGISTRY_PATH + json_file,
+                    reader=base.JsonPackagedFileReader(resource_package_name))
+                message_registries.append(mes_reg)
+
+        return message_registries
+
+    @property
+    @utils.cache_it
+    def registries(self):
+        """Gets and combines all message registries together
+
+        Fetches all registries if any provided by Redfish service
+        and combines together with packaged standard registries.
+
+        :returns: dict of combined message registries where key is
+            Registry_name.Major_version.Minor_version and value is registry
+            itself.
+        """
+        standard = self._get_standard_message_registry_collection()
+
+        registries = {r.registry_prefix + '.' +
+                      r.registry_version.rsplit('.', 1)[0]: r
+                      for r in standard if r.language == self._language}
+
+        registry_col = self._get_registry_collection()
+
+        if registry_col:
+            provided = registry_col.get_members()
+            registries.update({r.registry: r.get_message_registry(
+                               self._language,
+                               self._public_connector) for r in provided})
+
+        return registries
