@@ -15,10 +15,19 @@
 
 import abc
 import collections
+
+# (rpittau) this allows usage of collection ABC abstract classes in both
+# Python 2.7 and Python 3.8+
+try:
+    collectionsAbc = collections.abc
+except AttributeError:
+    collectionsAbc = collections
+
 import copy
 import io
 import json
 import logging
+import pkg_resources
 import zipfile
 
 import six
@@ -51,8 +60,9 @@ class Field(object):
         if not callable(adapter):
             raise TypeError("Adapter must be callable")
 
-        if isinstance(path, six.string_types):
+        if not isinstance(path, list):
             path = [path]
+
         elif not path:
             raise ValueError('Path cannot be empty')
 
@@ -61,6 +71,17 @@ class Field(object):
         self._default = default
         self._adapter = adapter
 
+    def _get_item(self, dct, key_or_callable, **context):
+        if not callable(key_or_callable):
+            return dct[key_or_callable]
+
+        for candidate_key in dct:
+            if key_or_callable(
+                    candidate_key, value=dct[candidate_key], **context):
+                return dct[candidate_key]
+
+        raise KeyError(key_or_callable)
+
     def _load(self, body, resource, nested_in=None):
         """Load this field from a JSON object.
 
@@ -76,7 +97,10 @@ class Field(object):
         for path_item in self._path[:-1]:
             body = body.get(path_item, {})
 
-        if name not in body:
+        try:
+            item = self._get_item(body, name)
+
+        except KeyError:
             if self._required:
                 path = (nested_in or []) + self._path
                 raise exceptions.MissingAttributeError(
@@ -87,7 +111,16 @@ class Field(object):
                 return self._default
 
         try:
-            value = self._adapter(body[name])
+            # Get the value based on the name, defaulting to an empty dict
+            # Check to ensure that value is implemented by OEM
+            # TODO(etingof): we should revisit this logic/code
+            if (item is not None and item != {} and
+               str(item).lower() != 'none'):
+                value = self._adapter(item)
+
+            else:
+                value = item
+
         except (UnicodeError, ValueError, TypeError) as exc:
             path = (nested_in or []) + self._path
             raise exceptions.MalformedAttributeError(
@@ -111,7 +144,7 @@ def _collect_fields(resource):
 
 
 @six.add_metaclass(abc.ABCMeta)
-class CompositeField(collections.Mapping, Field):
+class CompositeField(collectionsAbc.Mapping, Field):
     """Base class for fields consisting of several sub-fields."""
 
     def __init__(self, *args, **kwargs):
@@ -143,6 +176,7 @@ class CompositeField(collections.Mapping, Field):
 
     # Satisfy the mapping interface, see
     # https://docs.python.org/2/library/collections.html#collections.Mapping.
+    # https://docs.python.org/3/library/collections.abc.html#collections.abc.Mapping
 
     def __getitem__(self, key):
         if key in self._subfields:
@@ -240,7 +274,7 @@ class MappedField(Field):
             Only has effect when the field is not required. This value is not
             matched against the mapping.
         """
-        if not isinstance(mapping, collections.Mapping):
+        if not isinstance(mapping, collectionsAbc.Mapping):
             raise TypeError("The mapping argument must be a mapping")
 
         super(MappedField, self).__init__(
@@ -248,6 +282,52 @@ class MappedField(Field):
             adapter=mapping.get)
 
 
+class MappedListField(Field):
+    """Field taking a list of values with a mapping for the values
+
+    Given JSON {'field':['xxx', 'yyy']}, a sushy resource definition and
+    mapping {'xxx':'a', 'yyy':'b'}, the sushy object to come out will be like
+    resource.field = ['a', 'b']
+    """
+
+    def __init__(self, field, mapping, required=False, default=None):
+        """Create a mapped list field definition.
+
+        :param field: JSON field to fetch the list of values from.
+        :param mapping: a mapping for the list elements.
+        :param required: whether this field is required. Missing required
+            fields result in MissingAttributeError.
+        :param default: the default value to use when the field is missing.
+            Only has effect when the field is not required.
+        """
+        if not isinstance(mapping, collectionsAbc.Mapping):
+            raise TypeError("The mapping argument must be a mapping")
+
+        self._mapping_adapter = mapping.get
+        super(MappedListField, self).__init__(
+            field, required=required, default=default,
+            adapter=lambda x: x)
+
+    def _load(self, body, resource, nested_in=None):
+        """Load the mapped list.
+
+        :param body: parent JSON body.
+        :param resource: parent resource.
+        :param nested_in: parent resource name (for error reporting only).
+        :returns: a new list object containing the mapped values.
+        """
+        nested_in = (nested_in or []) + self._path
+        values = super(MappedListField, self)._load(body, resource)
+
+        if values is None:
+            return
+
+        instances = [self._mapping_adapter(value) for value in values
+                     if self._mapping_adapter(value) is not None]
+
+        return instances
+
+
 @six.add_metaclass(abc.ABCMeta)
 class AbstractJsonReader(object):
 
@@ -273,6 +353,14 @@ class JsonDataReader(AbstractJsonReader):
         return self._conn.get(path=self._path).json()
 
 
+class JsonPublicFileReader(AbstractJsonReader):
+    """Loads the data from the Internet"""
+
+    def get_json(self):
+        """Get JSON file from full URI"""
+        return self._conn.get(self._path).json()
+
+
 class JsonArchiveReader(AbstractJsonReader):
     """Gets the data from JSON file in archive"""
 
@@ -300,6 +388,24 @@ class JsonArchiveReader(AbstractJsonReader):
                       {'type': data.headers['content-type']})
 
 
+class JsonPackagedFileReader(AbstractJsonReader):
+    """Gets the data from packaged file given by path"""
+
+    def __init__(self, resource_package_name):
+        """Initializes the reader
+
+        :param resource_package: Python package/module name
+        """
+        self._resource_package_name = resource_package_name
+
+    def get_json(self):
+        """Gets JSON file from packaged file denoted by path"""
+
+        with pkg_resources.resource_stream(self._resource_package_name,
+                                           self._path) as resource:
+            return json.loads(resource.read().decode(encoding='utf-8'))
+
+
 @six.add_metaclass(abc.ABCMeta)
 class ResourceBase(object):
 
@@ -313,6 +419,7 @@ class ResourceBase(object):
                  connector,
                  path='',
                  redfish_version=None,
+                 registries=None,
                  reader=None):
         """A class representing the base of any Redfish resource
 
@@ -322,12 +429,15 @@ class ResourceBase(object):
         :param path: sub-URI path to the resource.
         :param redfish_version: The version of Redfish. Used to construct
             the object according to schema of the given version.
+        :param registries: Dict of Redfish Message Registry objects to be
+            used in any resource that needs registries to parse messages
         :param reader: Reader to use to fetch JSON data.
         """
         self._conn = connector
         self._path = path
         self._json = None
         self.redfish_version = redfish_version
+        self._registries = registries
         # Note(deray): Indicates if the resource holds stale data or not.
         # Starting off with True and eventually gets set to False when
         # attribute values are fetched.
@@ -340,11 +450,16 @@ class ResourceBase(object):
 
         self.refresh()
 
-    def _parse_attributes(self):
-        """Parse the attributes of a resource."""
+    def _parse_attributes(self, json_doc):
+        """Parse the attributes of a resource.
+
+        Parsed JSON fields are set to `self` as declared in the class.
+
+        :param json_doc: parsed JSON document in form of Python types
+        """
         for attr, field in _collect_fields(self):
             # Hide the Field object behind the real value
-            setattr(self, attr, field._load(self.json, self))
+            setattr(self, attr, field._load(json_doc, self))
 
     def refresh(self, force=True):
         """Refresh the resource
@@ -373,7 +488,7 @@ class ResourceBase(object):
         LOG.debug('Received representation of %(type)s %(path)s: %(json)s',
                   {'type': self.__class__.__name__,
                    'path': self._path, 'json': self._json})
-        self._parse_attributes()
+        self._parse_attributes(self._json)
         self._do_refresh(force)
 
         # Mark it fresh
@@ -425,6 +540,13 @@ class ResourceBase(object):
     def path(self):
         return self._path
 
+    def clone_resource(self, new_resource, path=''):
+        """Instantiate given resource using existing BMC connection context"""
+        return new_resource(
+            self._conn, path or self.path,
+            redfish_version=self.redfish_version,
+            reader=self._reader)
+
     @property
     def resource_name(self):
         return utils.camelcase_to_underscore_joined(self.__class__.__name__)
@@ -441,6 +563,10 @@ class ResourceBase(object):
         return oem.get_resource_extension_by_vendor(
             self.resource_name, vendor, self)
 
+    @property
+    def registries(self):
+        return self._registries
+
 
 @six.add_metaclass(abc.ABCMeta)
 class ResourceCollectionBase(ResourceBase):
@@ -452,7 +578,7 @@ class ResourceCollectionBase(ResourceBase):
                                adapter=utils.get_members_identities)
     """A tuple with the members identities"""
 
-    def __init__(self, connector, path, redfish_version=None):
+    def __init__(self, connector, path, redfish_version=None, registries=None):
         """A class representing the base of any Redfish resource collection
 
         It gets inherited from ``ResourceBase`` and invokes the base class
@@ -461,9 +587,11 @@ class ResourceCollectionBase(ResourceBase):
         :param path: sub-URI path to the resource collection.
         :param redfish_version: The version of Redfish. Used to construct
             the object according to schema of the given version.
+        :param registries: Dict of Redfish Message Registry objects to be
+            used in any resource that needs registries to parse messages.
         """
-        super(ResourceCollectionBase, self).__init__(connector, path,
-                                                     redfish_version)
+        super(ResourceCollectionBase, self).__init__(
+            connector, path, redfish_version, registries)
         LOG.debug('Received %(count)d member(s) for %(type)s %(path)s',
                   {'count': len(self.members_identities),
                    'type': self.__class__.__name__, 'path': self._path})
@@ -484,8 +612,8 @@ class ResourceCollectionBase(ResourceBase):
         :returns: The ``_resource_type`` object
         :raises: ResourceNotFoundError
         """
-        return self._resource_type(self._conn, identity,
-                                   redfish_version=self.redfish_version)
+        return self._resource_type(
+            self._conn, identity, self.redfish_version, self.registries)
 
     @utils.cache_it
     def get_members(self):
