@@ -24,6 +24,7 @@ from sushy.resources.chassis import chassis
 from sushy.resources import common
 from sushy.resources.manager import manager
 from sushy.resources import mappings as res_maps
+from sushy.resources import settings
 from sushy.resources.system import bios
 from sushy.resources.system import constants as sys_cons
 from sushy.resources.system import ethernet_interface
@@ -117,8 +118,8 @@ class System(base.ResourceBase):
     status = common.StatusField('Status')
     """The system status"""
 
-    # TODO(lucasagomes): Create mappings for the system_type
-    system_type = base.Field('SystemType')
+    system_type = base.MappedField('SystemType',
+                                   sys_maps.SYSTEM_TYPE_VALUE_MAP)
     """The system type"""
 
     uuid = base.Field('UUID')
@@ -127,17 +128,26 @@ class System(base.ResourceBase):
     memory_summary = MemorySummaryField('MemorySummary')
     """The summary info of memory of the system in general detail"""
 
+    maintenance_window = settings.MaintenanceWindowField(
+        '@Redfish.MaintenanceWindow')
+    """Indicates if a given resource has a maintenance window assignment
+    for applying settings or operations"""
+
     _actions = ActionsField('Actions', required=True)
 
-    def __init__(self, connector, identity, redfish_version=None):
+    def __init__(self, connector, identity, redfish_version=None,
+                 registries=None):
         """A class representing a ComputerSystem
 
         :param connector: A Connector instance
         :param identity: The identity of the System resource
         :param redfish_version: The version of RedFish. Used to construct
             the object according to schema of the given version.
+        :param registries: Dict of registries to be used in any resource
+            that needs registries to parse messages.
         """
-        super(System, self).__init__(connector, identity, redfish_version)
+        super(System, self).__init__(
+            connector, identity, redfish_version, registries)
 
     def _get_reset_action_element(self):
         reset_action = self._actions.reset
@@ -246,6 +256,26 @@ class System(base.ResourceBase):
         #                    Probably we should call refresh() as well.
         self._conn.patch(self.path, data=data)
 
+    def set_indicator_led(self, state):
+        """Set IndicatorLED to the given state.
+
+        :param state: Desired LED state, lit (INDICATOR_LED_LIT), blinking
+            (INDICATOR_LED_BLINKING), off (INDICATOR_LED_OFF)
+        :raises: InvalidParameterValueError, if any information passed is
+            invalid.
+        """
+        if state not in res_maps.INDICATOR_LED_VALUE_MAP_REV:
+            raise exceptions.InvalidParameterValueError(
+                parameter='state', value=state,
+                valid_values=list(res_maps.INDICATOR_LED_VALUE_MAP_REV))
+
+        data = {
+            'IndicatorLED': res_maps.INDICATOR_LED_VALUE_MAP_REV[state]
+        }
+
+        self._conn.patch(self.path, data=data)
+        self.invalidate()
+
     def _get_processor_collection_path(self):
         """Helper function to find the ProcessorCollection path"""
         return utils.get_sub_resource_path_by(self, 'Processors')
@@ -261,7 +291,8 @@ class System(base.ResourceBase):
         """
         return processor.ProcessorCollection(
             self._conn, self._get_processor_collection_path(),
-            redfish_version=self.redfish_version)
+            redfish_version=self.redfish_version,
+            registries=self.registries)
 
     @property
     @utils.cache_it
@@ -275,7 +306,8 @@ class System(base.ResourceBase):
         return ethernet_interface.EthernetInterfaceCollection(
             self._conn,
             utils.get_sub_resource_path_by(self, "EthernetInterfaces"),
-            redfish_version=self.redfish_version)
+            redfish_version=self.redfish_version,
+            registries=self.registries)
 
     @property
     @utils.cache_it
@@ -289,7 +321,8 @@ class System(base.ResourceBase):
         return bios.Bios(
             self._conn,
             utils.get_sub_resource_path_by(self, 'Bios'),
-            redfish_version=self.redfish_version)
+            redfish_version=self.redfish_version,
+            registries=self.registries)
 
     @property
     @utils.cache_it
@@ -310,7 +343,8 @@ class System(base.ResourceBase):
         """
         return sys_simple_storage.SimpleStorageCollection(
             self._conn, utils.get_sub_resource_path_by(self, "SimpleStorage"),
-            redfish_version=self.redfish_version)
+            redfish_version=self.redfish_version,
+            registries=self.registries)
 
     @property
     @utils.cache_it
@@ -332,7 +366,8 @@ class System(base.ResourceBase):
         """
         return sys_storage.StorageCollection(
             self._conn, utils.get_sub_resource_path_by(self, "Storage"),
-            redfish_version=self.redfish_version)
+            redfish_version=self.redfish_version,
+            registries=self.registries)
 
     @property
     @utils.cache_it
@@ -349,7 +384,8 @@ class System(base.ResourceBase):
             self, ["Links", "ManagedBy"], is_collection=True)
 
         return [manager.Manager(self._conn, path,
-                                redfish_version=self.redfish_version)
+                                redfish_version=self.redfish_version,
+                                registries=self.registries)
                 for path in paths]
 
     @property
@@ -367,7 +403,8 @@ class System(base.ResourceBase):
             self, ["Links", "Chassis"], is_collection=True)
 
         return [chassis.Chassis(self._conn, path,
-                                redfish_version=self.redfish_version)
+                                redfish_version=self.redfish_version,
+                                registries=self.registries)
                 for path in paths]
 
 
@@ -377,13 +414,15 @@ class SystemCollection(base.ResourceCollectionBase):
     def _resource_type(self):
         return System
 
-    def __init__(self, connector, path, redfish_version=None):
+    def __init__(self, connector, path, redfish_version=None, registries=None):
         """A class representing a ComputerSystemCollection
 
         :param connector: A Connector instance
         :param path: The canonical path to the System collection resource
         :param redfish_version: The version of RedFish. Used to construct
             the object according to schema of the given version.
+        :param registries: Dict of Redfish Message Registry objects to be
+            used in any resource that needs registries to parse messages
         """
-        super(SystemCollection, self).__init__(connector, path,
-                                               redfish_version)
+        super(SystemCollection, self).__init__(
+            connector, path, redfish_version, registries)
