@@ -29,14 +29,16 @@ class ManagerTestCase(base.TestCase):
         super(ManagerTestCase, self).setUp()
         self.conn = mock.Mock()
         with open('sushy/tests/unit/json_samples/manager.json') as f:
-            self.conn.get.return_value.json.return_value = json.load(f)
+            self.json_doc = json.load(f)
+
+        self.conn.get.return_value.json.return_value = self.json_doc
 
         self.manager = manager.Manager(self.conn, '/redfish/v1/Managers/BMC',
                                        redfish_version='1.0.2')
 
     def test__parse_attributes(self):
         # | WHEN |
-        self.manager._parse_attributes()
+        self.manager._parse_attributes(self.json_doc)
         # | THEN |
         self.assertEqual('1.0.2', self.manager.redfish_version)
         self.assertEqual('1.00', self.manager.firmware_version)
@@ -309,13 +311,13 @@ class ManagerCollectionTestCase(base.TestCase):
         self.managers.get_member('/redfish/v1/Managers/BMC')
         Manager_mock.assert_called_once_with(
             self.managers._conn, '/redfish/v1/Managers/BMC',
-            redfish_version=self.managers.redfish_version)
+            self.managers.redfish_version, None)
 
     @mock.patch.object(manager, 'Manager', autospec=True)
     def test_get_members(self, Manager_mock):
         members = self.managers.get_members()
         Manager_mock.assert_called_once_with(
             self.managers._conn, '/redfish/v1/Managers/BMC',
-            redfish_version=self.managers.redfish_version)
+            self.managers.redfish_version, None)
         self.assertIsInstance(members, list)
         self.assertEqual(1, len(members))
