@@ -13,10 +13,14 @@
 #    under the License.
 
 import json
-
 import mock
 
+from dateutil import parser
+from six.moves import http_client
+
 from sushy import exceptions
+from sushy.resources.registry import message_registry
+from sushy.resources import settings
 from sushy.resources.system import bios
 from sushy.tests.unit import base
 
@@ -27,21 +31,29 @@ class BiosTestCase(base.TestCase):
         super(BiosTestCase, self).setUp()
         self.conn = mock.Mock()
         with open('sushy/tests/unit/json_samples/bios.json') as f:
-            bios_json = json.load(f)
+            self.bios_json = json.load(f)
         with open('sushy/tests/unit/json_samples/bios_settings.json') as f:
-            bios_settings_json = json.load(f)
+            self.bios_settings_json = json.load(f)
 
         self.conn.get.return_value.json.side_effect = [
-            bios_json,
-            bios_settings_json,
-            bios_settings_json]
+            self.bios_json,
+            self.bios_settings_json,
+            self.bios_settings_json]
+
+        conn = mock.Mock()
+        with open('sushy/tests/unit/json_samples/message_registry.json') as f:
+            conn.get.return_value.json.return_value = json.load(f)
+            registry = message_registry.MessageRegistry(
+                conn, '/redfish/v1/Registries/Test',
+                redfish_version='1.0.2')
 
         self.sys_bios = bios.Bios(
             self.conn, '/redfish/v1/Systems/437XR1138R2/BIOS',
+            registries={'Test.1.0': registry},
             redfish_version='1.0.2')
 
     def test__parse_attributes(self):
-        self.sys_bios._parse_attributes()
+        self.sys_bios._parse_attributes(self.bios_json)
         self.assertEqual('1.0.2', self.sys_bios.redfish_version)
         self.assertEqual('BIOS', self.sys_bios.identity)
         self.assertEqual('BIOS Configuration Current Settings',
@@ -58,6 +70,8 @@ class BiosTestCase(base.TestCase):
                          self.sys_bios._settings._etag)
         self.assertEqual('(404) 555-1212',
                          self.sys_bios.pending_attributes['AdminPhone'])
+        self.assertEqual(settings.UPDATE_FAILURE,
+                         self.sys_bios.update_status.status)
 
     def test_set_attribute(self):
         self.sys_bios.set_attribute('ProcTurboMode', 'Disabled')
@@ -106,6 +120,19 @@ class BiosTestCase(base.TestCase):
         self.sys_bios.pending_attributes
         self.assertTrue(self.conn.get.called)
 
+    def test_apply_time_settings(self):
+        self.conn.get.reset_mock()
+        apply_time_settings = self.sys_bios.apply_time_settings
+        self.assertIsNotNone(apply_time_settings)
+        self.assertEqual('OnReset', apply_time_settings.apply_time)
+        self.assertEqual(['OnReset', 'Immediate', 'AtMaintenanceWindowStart',
+                         'InMaintenanceWindowOnReset'],
+                         apply_time_settings.apply_time_allowable_values)
+        self.assertEqual(parser.parse('2017-05-03T23:12:37-05:00'),
+                         apply_time_settings.maintenance_window_start_time)
+        self.assertEqual(600, apply_time_settings.
+                         maintenance_window_duration_in_seconds)
+
     def test__get_reset_bios_action_element(self):
         value = self.sys_bios._get_reset_bios_action_element()
         self.assertEqual('/redfish/v1/Systems/437XR1138R2/BIOS/Actions/'
@@ -124,13 +151,37 @@ class BiosTestCase(base.TestCase):
         self.assertRaisesRegex(
             exceptions.MissingAttributeError,
             'attribute Actions/#Bios.ResetBios/target',
-            self.sys_bios._parse_attributes)
+            self.sys_bios._parse_attributes, self.bios_json)
 
     def test_reset_bios(self):
         self.sys_bios.reset_bios()
         self.sys_bios._conn.post.assert_called_once_with(
             '/redfish/v1/Systems/437XR1138R2/BIOS/Actions/Bios.ResetBios')
 
+    def test_reset_bios_handle_http_error_415(self):
+
+        target_uri = (
+            '/redfish/v1/Systems/437XR1138R2/BIOS/Actions/Bios.ResetBios')
+        self.conn.post.side_effect = [exceptions.HTTPError(
+            method='POST', url=target_uri, response=mock.MagicMock(
+                status_code=http_client.UNSUPPORTED_MEDIA_TYPE)), '200']
+        post_calls = [
+            mock.call(target_uri), mock.call(target_uri, data={})]
+        self.sys_bios.reset_bios()
+        self.sys_bios._conn.post.assert_has_calls(post_calls)
+
+    def test_reset_bios_handle_http_error_405(self):
+
+        target_uri = (
+            '/redfish/v1/Systems/437XR1138R2/BIOS/Actions/Bios.ResetBios')
+        self.conn.post.side_effect = exceptions.HTTPError(
+            method='POST', url=target_uri, response=mock.MagicMock(
+                status_code=http_client.METHOD_NOT_ALLOWED))
+        self.assertRaises(
+            exceptions.HTTPError,
+            self.sys_bios.reset_bios)
+        self.sys_bios._conn.post.assert_called_once_with(target_uri)
+
     def test__get_change_password_element(self):
         value = self.sys_bios._get_change_password_element()
         self.assertEqual("/redfish/v1/Systems/437XR1138R2/BIOS/Actions/"
@@ -151,7 +202,7 @@ class BiosTestCase(base.TestCase):
         self.assertRaisesRegex(
             exceptions.MissingAttributeError,
             'attribute Actions/#Bios.ChangePassword/target',
-            self.sys_bios._parse_attributes)
+            self.sys_bios._parse_attributes, self.bios_json)
 
     def test_change_password(self):
         self.sys_bios.change_password('newpassword',
