@@ -14,15 +14,21 @@
 #    under the License.
 
 import json
-
 import mock
 
+from dateutil import parser
+
 import sushy
 from sushy import exceptions
-from sushy.resources.system import constants as sys_cons
-from sushy.resources.system import ethernet_interface
+from sushy.resources.chassis import chassis
+from sushy.resources import constants as res_cons
+from sushy.resources.manager import manager
+from sushy.resources.oem import fake
+from sushy.resources.system import bios
 from sushy.resources.system import mappings as sys_map
 from sushy.resources.system import processor
+from sushy.resources.system import simple_storage
+from sushy.resources.system.storage import storage
 from sushy.resources.system import system
 from sushy.tests.unit import base
 
@@ -32,48 +38,62 @@ class SystemTestCase(base.TestCase):
     def setUp(self):
         super(SystemTestCase, self).setUp()
         self.conn = mock.Mock()
-        with open('sushy/tests/unit/json_samples/system.json', 'r') as f:
-            self.conn.get.return_value.json.return_value = json.loads(f.read())
+        with open('sushy/tests/unit/json_samples/system.json') as f:
+            self.json_doc = json.load(f)
+
+        self.conn.get.return_value.json.return_value = self.json_doc
 
         self.sys_inst = system.System(
             self.conn, '/redfish/v1/Systems/437XR1138R2',
             redfish_version='1.0.2')
 
     def test__parse_attributes(self):
-        self.sys_inst._parse_attributes()
+        self.sys_inst._parse_attributes(self.json_doc)
         self.assertEqual('1.0.2', self.sys_inst.redfish_version)
         self.assertEqual('Chicago-45Z-2381', self.sys_inst.asset_tag)
         self.assertEqual('P79 v1.33 (02/28/2015)', self.sys_inst.bios_version)
         self.assertEqual('Web Front End node', self.sys_inst.description)
         self.assertEqual('web483', self.sys_inst.hostname)
         self.assertEqual('437XR1138R2', self.sys_inst.identity)
-        self.assertEqual('Off', self.sys_inst.indicator_led)
+        self.assertEqual(sushy.INDICATOR_LED_OFF,
+                         self.sys_inst.indicator_led)
         self.assertEqual('Contoso', self.sys_inst.manufacturer)
         self.assertEqual('WebFrontEnd483', self.sys_inst.name)
         self.assertEqual('224071-J23', self.sys_inst.part_number)
         self.assertEqual('437XR1138R2', self.sys_inst.serial_number)
         self.assertEqual('8675309', self.sys_inst.sku)
-        self.assertEqual('Physical', self.sys_inst.system_type)
+        self.assertEqual(sushy.SYSTEM_TYPE_PHYSICAL,
+                         self.sys_inst.system_type)
         self.assertEqual('38947555-7742-3448-3784-823347823834',
                          self.sys_inst.uuid)
+        self.assertEqual(res_cons.STATE_ENABLED, self.sys_inst.status.state)
+        self.assertEqual(res_cons.HEALTH_OK, self.sys_inst.status.health)
+        self.assertEqual(res_cons.HEALTH_OK,
+                         self.sys_inst.status.health_rollup)
         self.assertEqual(sushy.SYSTEM_POWER_STATE_ON,
                          self.sys_inst.power_state)
         self.assertEqual(96, self.sys_inst.memory_summary.size_gib)
         self.assertEqual("OK", self.sys_inst.memory_summary.health)
-        self.assertIsNone(self.sys_inst._processors)
-        self.assertIsNone(self.sys_inst._ethernet_interfaces)
+        self.assertIsNotNone(self.sys_inst.maintenance_window)
+        self.assertEqual(1, self.sys_inst.maintenance_window
+                         .maintenance_window_duration_in_seconds)
+        self.assertEqual(parser.parse('2016-03-07T14:44:30-05:05'),
+                         self.sys_inst.maintenance_window
+                         .maintenance_window_start_time)
+        for oem_vendor in self.sys_inst.oem_vendors:
+            self.assertIn(oem_vendor, ('Contoso', 'Chipwise'))
 
     def test__parse_attributes_missing_actions(self):
         self.sys_inst.json.pop('Actions')
         self.assertRaisesRegex(
             exceptions.MissingAttributeError, 'attribute Actions',
-            self.sys_inst._parse_attributes)
+            self.sys_inst._parse_attributes, self.json_doc)
 
     def test__parse_attributes_missing_boot(self):
         self.sys_inst.json.pop('Boot')
         self.assertRaisesRegex(
             exceptions.MissingAttributeError, 'attribute Boot',
-            self.sys_inst._parse_attributes)
+            self.sys_inst._parse_attributes, self.json_doc)
 
     def test__parse_attributes_missing_reset_target(self):
         self.sys_inst.json['Actions']['#ComputerSystem.Reset'].pop(
@@ -81,13 +101,21 @@ class SystemTestCase(base.TestCase):
         self.assertRaisesRegex(
             exceptions.MissingAttributeError,
             'attribute Actions/#ComputerSystem.Reset/target',
-            self.sys_inst._parse_attributes)
+            self.sys_inst._parse_attributes, self.json_doc)
 
     def test__parse_attributes_null_memory_capacity(self):
         self.sys_inst.json['MemorySummary']['TotalSystemMemoryGiB'] = None
-        self.sys_inst._parse_attributes()
+        self.sys_inst._parse_attributes(self.json_doc)
         self.assertIsNone(self.sys_inst.memory_summary.size_gib)
 
+    def test__parse_attributes_bad_maintenance_window_time(self):
+        self.sys_inst.json['@Redfish.MaintenanceWindow'][
+            'MaintenanceWindowStartTime'] = 'bad date'
+        self.assertRaisesRegex(
+            exceptions.MalformedAttributeError,
+            '@Redfish.MaintenanceWindow/MaintenanceWindowStartTime',
+            self.sys_inst._parse_attributes, self.json_doc)
+
     def test_get__reset_action_element(self):
         value = self.sys_inst._get_reset_action_element()
         self.assertEqual("/redfish/v1/Systems/437XR1138R2/Actions/"
@@ -139,6 +167,17 @@ class SystemTestCase(base.TestCase):
         self.assertIsInstance(values, set)
         self.assertEqual(1, mock_log.call_count)
 
+    def test_reset_action_operation_apply_time_support(self):
+        support = self.sys_inst._actions.reset.operation_apply_time_support
+        self.assertIsNotNone(support)
+        self.assertEqual(['Immediate', 'AtMaintenanceWindowStart'],
+                         support.supported_values)
+        self.assertEqual(parser.parse('2017-05-03T23:12:37-05:00'),
+                         support.maintenance_window_start_time)
+        self.assertEqual(600, support.maintenance_window_duration_in_seconds)
+        self.assertEqual('/redfish/v1/Systems/437XR1138R2',
+                         support._maintenance_window_resource.resource_uri)
+
     def test_reset_system(self):
         self.sys_inst.reset_system(sushy.RESET_FORCE_OFF)
         self.sys_inst._conn.post.assert_called_once_with(
@@ -213,10 +252,29 @@ class SystemTestCase(base.TestCase):
                           'invalid-target')
 
     def test_set_system_boot_source_invalid_enabled(self):
+        with self.assertRaisesRegex(
+            exceptions.InvalidParameterValueError,
+            '"enabled" value.*{0}'.format(
+                list(sys_map.BOOT_SOURCE_ENABLED_MAP_REV))):
+
+            self.sys_inst.set_system_boot_source(
+                sushy.BOOT_SOURCE_TARGET_HDD,
+                enabled='invalid-enabled')
+
+    def test_set_indicator_led(self):
+        with mock.patch.object(
+                self.sys_inst, 'invalidate', autospec=True) as invalidate_mock:
+            self.sys_inst.set_indicator_led(sushy.INDICATOR_LED_BLINKING)
+            self.sys_inst._conn.patch.assert_called_once_with(
+                '/redfish/v1/Systems/437XR1138R2',
+                data={'IndicatorLED': 'Blinking'})
+
+            invalidate_mock.assert_called_once_with()
+
+    def test_set_indicator_led_invalid_state(self):
         self.assertRaises(exceptions.InvalidParameterValueError,
-                          self.sys_inst.set_system_boot_source,
-                          sushy.BOOT_SOURCE_TARGET_HDD,
-                          enabled='invalid-enabled')
+                          self.sys_inst.set_indicator_led,
+                          'spooky-glowing')
 
     def test__get_processor_collection_path_missing_processors_attr(self):
         self.sys_inst._json.pop('Processors')
@@ -228,7 +286,7 @@ class SystemTestCase(base.TestCase):
         # | GIVEN |
         self.sys_inst._json['MemorySummary']['Status'].pop('HealthRollup')
         # | WHEN |
-        self.sys_inst._parse_attributes()
+        self.sys_inst._parse_attributes(self.json_doc)
         # | THEN |
         self.assertEqual(96, self.sys_inst.memory_summary.size_gib)
         self.assertIsNone(self.sys_inst.memory_summary.health)
@@ -236,7 +294,7 @@ class SystemTestCase(base.TestCase):
         # | GIVEN |
         self.sys_inst._json['MemorySummary'].pop('Status')
         # | WHEN |
-        self.sys_inst._parse_attributes()
+        self.sys_inst._parse_attributes(self.json_doc)
         # | THEN |
         self.assertEqual(96, self.sys_inst.memory_summary.size_gib)
         self.assertIsNone(self.sys_inst.memory_summary.health)
@@ -244,7 +302,7 @@ class SystemTestCase(base.TestCase):
         # | GIVEN |
         self.sys_inst._json['MemorySummary'].pop('TotalSystemMemoryGiB')
         # | WHEN |
-        self.sys_inst._parse_attributes()
+        self.sys_inst._parse_attributes(self.json_doc)
         # | THEN |
         self.assertIsNone(self.sys_inst.memory_summary.size_gib)
         self.assertIsNone(self.sys_inst.memory_summary.health)
@@ -252,18 +310,16 @@ class SystemTestCase(base.TestCase):
         # | GIVEN |
         self.sys_inst._json.pop('MemorySummary')
         # | WHEN |
-        self.sys_inst._parse_attributes()
+        self.sys_inst._parse_attributes(self.json_doc)
         # | THEN |
         self.assertIsNone(self.sys_inst.memory_summary)
 
     def test_processors(self):
-        # check for the underneath variable value
-        self.assertIsNone(self.sys_inst._processors)
         # | GIVEN |
         self.conn.get.return_value.json.reset_mock()
-        with open('sushy/tests/unit/json_samples/processor_collection.json',
-                  'r') as f:
-            self.conn.get.return_value.json.return_value = json.loads(f.read())
+        with open('sushy/tests/unit/json_samples/'
+                  'processor_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
         # | WHEN |
         actual_processors = self.sys_inst.processors
         # | THEN |
@@ -281,47 +337,43 @@ class SystemTestCase(base.TestCase):
 
     def test_processors_on_refresh(self):
         # | GIVEN |
-        with open('sushy/tests/unit/json_samples/processor_collection.json',
-                  'r') as f:
-            self.conn.get.return_value.json.return_value = json.loads(f.read())
+        with open('sushy/tests/unit/json_samples/'
+                  'processor_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
         # | WHEN & THEN |
         self.assertIsInstance(self.sys_inst.processors,
                               processor.ProcessorCollection)
 
         # On refreshing the system instance...
-        with open('sushy/tests/unit/json_samples/system.json', 'r') as f:
-            self.conn.get.return_value.json.return_value = json.loads(f.read())
+        with open('sushy/tests/unit/json_samples/system.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
 
         self.sys_inst.invalidate()
         self.sys_inst.refresh(force=False)
 
-        # | WHEN & THEN |
-        self.assertIsNotNone(self.sys_inst._processors)
-        self.assertTrue(self.sys_inst._processors._is_stale)
-
         # | GIVEN |
-        with open('sushy/tests/unit/json_samples/processor_collection.json',
-                  'r') as f:
-            self.conn.get.return_value.json.return_value = json.loads(f.read())
+        with open('sushy/tests/unit/json_samples/'
+                  'processor_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
         # | WHEN & THEN |
         self.assertIsInstance(self.sys_inst.processors,
                               processor.ProcessorCollection)
-        self.assertFalse(self.sys_inst._processors._is_stale)
 
     def _setUp_processor_summary(self):
         self.conn.get.return_value.json.reset_mock()
-        with open('sushy/tests/unit/json_samples/processor_collection.json',
-                  'r') as f:
-            self.conn.get.return_value.json.return_value = json.loads(f.read())
+        with open('sushy/tests/unit/json_samples/'
+                  'processor_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
 
         # fetch processors for the first time
         self.sys_inst.processors
 
         successive_return_values = []
-        with open('sushy/tests/unit/json_samples/processor.json', 'r') as f:
-            successive_return_values.append(json.loads(f.read()))
-        with open('sushy/tests/unit/json_samples/processor2.json', 'r') as f:
-            successive_return_values.append(json.loads(f.read()))
+        file_names = ['sushy/tests/unit/json_samples/processor.json',
+                      'sushy/tests/unit/json_samples/processor2.json']
+        for file_name in file_names:
+            with open(file_name) as f:
+                successive_return_values.append(json.load(f))
 
         self.conn.get.return_value.json.side_effect = successive_return_values
 
@@ -350,26 +402,163 @@ class SystemTestCase(base.TestCase):
         self.conn.get.return_value.json.reset_mock()
         eth_coll_return_value = None
         eth_return_value = None
-        path = ('sushy/tests/unit/json_samples/'
-                'ethernet_interfaces_collection.json')
-        with open(path, 'r') as f:
-            eth_coll_return_value = json.loads(f.read())
-        with open('sushy/tests/unit/json_samples/ethernet_interfaces.json',
-                  'r') as f:
-            eth_return_value = (json.loads(f.read()))
+        with open('sushy/tests/unit/json_samples/'
+                  'ethernet_interfaces_collection.json') as f:
+            eth_coll_return_value = json.load(f)
+        with open('sushy/tests/unit/json_samples/'
+                  'ethernet_interfaces.json') as f:
+            eth_return_value = json.load(f)
 
         self.conn.get.return_value.json.side_effect = [eth_coll_return_value,
                                                        eth_return_value]
 
-        self.assertIsNone(self.sys_inst._ethernet_interfaces)
         actual_macs = self.sys_inst.ethernet_interfaces.summary
         expected_macs = (
-            {'12:44:6A:3B:04:11':
-             sys_map.HEALTH_STATE_VALUE_MAP_REV.get(
-                 sys_cons.HEALTH_STATE_ENABLED)})
+            {'12:44:6A:3B:04:11': res_cons.STATE_ENABLED})
         self.assertEqual(expected_macs, actual_macs)
-        self.assertIsInstance(self.sys_inst._ethernet_interfaces,
-                              ethernet_interface.EthernetInterfaceCollection)
+
+    def test_bios(self):
+        self.conn.get.return_value.json.reset_mock()
+        bios_return_value = None
+        with open('sushy/tests/unit/json_samples/bios.json') as f:
+            bios_return_value = json.load(f)
+        self.conn.get.return_value.json.side_effect = [bios_return_value]
+
+        self.assertIsInstance(self.sys_inst.bios, bios.Bios)
+        self.assertEqual('BIOS Configuration Current Settings',
+                         self.sys_inst.bios.name)
+
+    def test_simple_storage_for_missing_attr(self):
+        self.sys_inst.json.pop('SimpleStorage')
+        with self.assertRaisesRegex(
+                exceptions.MissingAttributeError, 'attribute SimpleStorage'):
+            self.sys_inst.simple_storage
+
+    def test_simple_storage(self):
+        # | GIVEN |
+        self.conn.get.return_value.json.reset_mock()
+        with open('sushy/tests/unit/json_samples/'
+                  'simple_storage_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+        # | WHEN |
+        actual_simple_storage = self.sys_inst.simple_storage
+        # | THEN |
+        self.assertIsInstance(actual_simple_storage,
+                              simple_storage.SimpleStorageCollection)
+        self.conn.get.return_value.json.assert_called_once_with()
+
+        # reset mock
+        self.conn.get.return_value.json.reset_mock()
+        # | WHEN & THEN |
+        # tests for same object on invoking subsequently
+        self.assertIs(actual_simple_storage,
+                      self.sys_inst.simple_storage)
+        self.conn.get.return_value.json.assert_not_called()
+
+    def test_simple_storage_on_refresh(self):
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'simple_storage_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+        # | WHEN & THEN |
+        self.assertIsInstance(self.sys_inst.simple_storage,
+                              simple_storage.SimpleStorageCollection)
+
+        # On refreshing the system instance...
+        with open('sushy/tests/unit/json_samples/system.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+
+        self.sys_inst.invalidate()
+        self.sys_inst.refresh(force=False)
+
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'simple_storage_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+        # | WHEN & THEN |
+        self.assertIsInstance(self.sys_inst.simple_storage,
+                              simple_storage.SimpleStorageCollection)
+
+    def test_storage_for_missing_attr(self):
+        self.sys_inst.json.pop('Storage')
+        with self.assertRaisesRegex(
+                exceptions.MissingAttributeError, 'attribute Storage'):
+            self.sys_inst.storage
+
+    def test_storage(self):
+        # | GIVEN |
+        self.conn.get.return_value.json.reset_mock()
+        with open('sushy/tests/unit/json_samples/'
+                  'storage_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+        # | WHEN |
+        actual_storage = self.sys_inst.storage
+        # | THEN |
+        self.assertIsInstance(actual_storage, storage.StorageCollection)
+        self.conn.get.return_value.json.assert_called_once_with()
+
+        # reset mock
+        self.conn.get.return_value.json.reset_mock()
+        # | WHEN & THEN |
+        # tests for same object on invoking subsequently
+        self.assertIs(actual_storage, self.sys_inst.storage)
+        self.conn.get.return_value.json.assert_not_called()
+
+    def test_storage_on_refresh(self):
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'storage_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+        # | WHEN & THEN |
+        self.assertIsInstance(self.sys_inst.storage,
+                              storage.StorageCollection)
+
+        # On refreshing the system instance...
+        with open('sushy/tests/unit/json_samples/system.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+
+        self.sys_inst.invalidate()
+        self.sys_inst.refresh(force=False)
+
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'storage_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+        # | WHEN & THEN |
+        self.assertIsInstance(self.sys_inst.storage, storage.StorageCollection)
+
+    def test_managers(self):
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'manager.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+
+        # | WHEN & THEN |
+        actual_managers = self.sys_inst.managers
+        self.assertIsInstance(actual_managers[0], manager.Manager)
+        self.assertEqual(
+            '/redfish/v1/Managers/BMC', actual_managers[0].path)
+
+    def test_chassis(self):
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'chassis.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+
+        # | WHEN & THEN |
+        actual_chassis = self.sys_inst.chassis
+        self.assertIsInstance(actual_chassis[0], chassis.Chassis)
+        self.assertEqual(
+            '/redfish/v1/Chassis/1U', actual_chassis[0].path)
+
+    def test_get_oem_extension(self):
+        # | WHEN |
+        contoso_system_extn_inst = self.sys_inst.get_oem_extension('Contoso')
+        # | THEN |
+        self.assertIsInstance(contoso_system_extn_inst,
+                              fake.FakeOEMSystemExtension)
+        self.assertIs(self.sys_inst, contoso_system_extn_inst._parent_resource)
+        self.assertEqual('Contoso', contoso_system_extn_inst._vendor_id)
 
 
 class SystemCollectionTestCase(base.TestCase):
@@ -378,13 +567,16 @@ class SystemCollectionTestCase(base.TestCase):
         super(SystemCollectionTestCase, self).setUp()
         self.conn = mock.Mock()
         with open('sushy/tests/unit/json_samples/'
-                  'system_collection.json', 'r') as f:
-            self.conn.get.return_value.json.return_value = json.loads(f.read())
+                  'system_collection.json') as f:
+            self.json_doc = json.load(f)
+
+        self.conn.get.return_value.json.return_value = self.json_doc
+
         self.sys_col = system.SystemCollection(
             self.conn, '/redfish/v1/Systems', redfish_version='1.0.2')
 
     def test__parse_attributes(self):
-        self.sys_col._parse_attributes()
+        self.sys_col._parse_attributes(self.json_doc)
         self.assertEqual('1.0.2', self.sys_col.redfish_version)
         self.assertEqual('Computer System Collection', self.sys_col.name)
         self.assertEqual(('/redfish/v1/Systems/437XR1138R2',),
@@ -395,13 +587,13 @@ class SystemCollectionTestCase(base.TestCase):
         self.sys_col.get_member('/redfish/v1/Systems/437XR1138R2')
         mock_system.assert_called_once_with(
             self.sys_col._conn, '/redfish/v1/Systems/437XR1138R2',
-            redfish_version=self.sys_col.redfish_version)
+            self.sys_col.redfish_version, None)
 
     @mock.patch.object(system, 'System', autospec=True)
     def test_get_members(self, mock_system):
         members = self.sys_col.get_members()
         mock_system.assert_called_once_with(
             self.sys_col._conn, '/redfish/v1/Systems/437XR1138R2',
-            redfish_version=self.sys_col.redfish_version)
+            self.sys_col.redfish_version, None)
         self.assertIsInstance(members, list)
         self.assertEqual(1, len(members))
